package com.absinthe.libchecker.features.snapshot.ui

import android.content.DialogInterface
import com.absinthe.libchecker.R
import com.absinthe.libchecker.constant.GlobalValues
import com.absinthe.libchecker.constant.options.SnapshotOptions
import com.absinthe.libchecker.features.snapshot.ui.view.SnapshotMenuBSDView
import com.absinthe.libchecker.features.snapshot.ui.view.SnapshotMenuItemView
import com.absinthe.libraries.utils.base.BaseBottomSheetViewDialogFragment
import com.absinthe.libraries.utils.view.BottomSheetHeaderView

class SnapshotMenuBSDFragment : BaseBottomSheetViewDialogFragment<SnapshotMenuBSDView>() {

  private val previousAdvancedOptions = GlobalValues.snapshotOptions
  private val optionsViewMap = mutableMapOf<Int, SnapshotMenuItemView>()

  private var onDismissCallback: () -> Unit = {}

  override fun initRootView(): SnapshotMenuBSDView = SnapshotMenuBSDView(requireContext())

  override fun getHeaderView(): BottomSheetHeaderView = root.getHeaderView()

  override fun init() {
    root.post {
      maxPeekSize = ((dialog?.window?.decorView?.height ?: 0) * 0.8).toInt()
    }
    optionsViewMap[SnapshotOptions.SHOW_UPDATE_TIME] = root.addOptionItemView(R.string.snapshot_menu_show_update_time, SnapshotOptions.SHOW_UPDATE_TIME)
    optionsViewMap[SnapshotOptions.HIDE_NO_COMPONENT_CHANGES] = root.addOptionItemView(R.string.snapshot_menu_hide_no_component_changes, SnapshotOptions.HIDE_NO_COMPONENT_CHANGES)
    optionsViewMap[SnapshotOptions.DIFF_HIGHLIGHT] = root.addOptionItemView(R.string.snapshot_menu_diff_highlight, SnapshotOptions.DIFF_HIGHLIGHT)

    optionsViewMap[SnapshotOptions.SHOW_UPDATE_TIME]?.setOnCheckedChangeCallback {
      root.updateDemoView()
    }
    optionsViewMap[SnapshotOptions.HIDE_NO_COMPONENT_CHANGES]?.setOnCheckedChangeCallback {
      root.updateDemoView()
    }
    optionsViewMap[SnapshotOptions.DIFF_HIGHLIGHT]?.setOnCheckedChangeCallback {
      root.updateDemoView()
    }

    dialog?.setOnDismissListener {
      if (GlobalValues.snapshotOptions != previousAdvancedOptions) {
        onDismissCallback()
      }
    }
  }

  override fun onDestroyView() {
    super.onDestroyView()
    optionsViewMap.clear()
  }

  override fun onCancel(dialog: DialogInterface) {
    super.onCancel(dialog)
    runCatching {
      dismiss()
    }
  }

  fun setOnDismissListener(action: () -> Unit) {
    onDismissCallback = action
  }
}
